<?php

namespace App\Http\Controllers\Backend;

use App\Helpers\GeneralHelper;
use App\Http\Controllers\Controller;

use App\Models\Album;
use App\Models\Photo;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class AlbumController extends Controller
{
    public function index(){
        $albums = Album::get();
        return view('backend.album.index',['data' => $albums]);
    }

    public function show(){
        return view('backend.album.show');
    }

    public function create(){
        return view('backend.album.add');
    }

    public function store(Request $request){
        $data = $request->validate([
            'title' => 'required|string|max:255',
            'upload_date' => 'string'
        ]);
        $data['sort_order'] = request()->sort_order??1;
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        $data['slug'] = $this->generate_slug($data['title']);

        Album::create($data);
        return redirect()->route('admin.album.index')->with('success','Album created successfully.');
    }

    public function edit(Album $album)
    {
        return view('backend.album.edit',['data' => $album]);
    }

    public function update(Request $request,$id)
    {
        $data = $request->validate([
            'title' => 'required|string|max:255',
            'upload_date' => 'string'
        ]);
        $data['slug'] = $this->generate_slug($data['title']);
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        Album::where('id',$id)->update($data);
        return redirect()->route('admin.album.index')->with('success','Album has been updated successfully.');
        // return redirect()->route('manageAlbums')->with('success', 'Album updated successfully');
    }

    public function destroy($id)
    {
        $album = Album::where('id', $id)->whereHas('photos')->first();
        if($album->photos){
            return redirect()->route('admin.album.index')->with('error','Album cannot be deleted. Please delete all photos in the Album first.');
        }
        Album::find($id)->delete();
        return redirect()->route('admin.album.index')->with('success','Album has been deleted successfully.');

    }
    

    public function getAlbumImages($albumId){
        $photos = Photo::where('album_id',$albumId)->orderBy('clickedDate', 'asc')->get();
        $album = Album::where('id',$albumId)->first();
        return view('photos.managePhotos',compact('photos','albumId','album'));
    }

    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }
}
