<?php

namespace App\Http\Controllers\Backend;

use App\Helpers\GeneralHelper;
use App\Http\Controllers\Controller;

use App\Models\Album;
use App\Models\Download;
use App\Models\DownloadCategory;
use App\Models\Photo;
use App\Traits\UploadTrait;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class DownloadController extends Controller
{
    use UploadTrait;

    private $fileUploadLocation = '/uploads/downloads/';

    public function index(){
        $albums = Download::get();
        return view('backend.download.index',['data' => $albums]);
    }

    public function show(){
        return view('backend.download');
    }

    public function create(){
        $data['category'] = DownloadCategory::where('active',1)->orderBy('sort_order','ASC')->get();
        return view('backend.download.add',$data);
    }

    public function store(Request $request){
        $data = $request->validate([
            'category_id' => 'sometimes',
            'title' => 'required|string|max:255',
            'file' => 'required|mimes:jpg,bmp,png,jpeg,doc,docx,xls,xlsx,pdf',
            'uploaded_date' => 'required',
            'sort_order' => ''
        ]);
        $data['slug'] = $this->generate_slug($data['title']);

        $data['sort_order'] = request()->sort_order??1;

        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        if (request()->hasFile('file')) {
            $data['file'] = $this->uploadFile(request()->file('file'), $this->fileUploadLocation);
        }
        Download::create($data);
        return redirect()->route('admin.downloads.index')->with('success','Download created successfully.');
    }

    public function edit(Download $download)
    {
        $data['category'] = DownloadCategory::where('active',1)->orderBy('sort_order','ASC')->get();
        $data['data'] = $download;
        return view('backend.download.edit',$data);
    }

    public function update(Request $request,$id)
    {
        $data = $request->validate([
            'category_id' => 'sometimes',
            'title' => 'required|string|max:255',
            'file' => 'mimes:jpg,bmp,png,jpeg,doc,docx,xls,xlsx,pdf',
            'uploaded_date' => 'required',
            'sort_order' => ''
        ]);
        $data['sort_order'] = request()->sort_order??1;
        $data['slug'] = $this->generate_slug($data['title']);

        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }

        if (request()->hasFile('file')) {
            $request->validate([
                'file' => 'mimes:jpg,bmp,png,jpeg,doc,docx,xls,xlsx,pdf'
            ]);
            $data['file'] = $this->uploadOne(request()->file('file'), $this->fileUploadLocation);
            // if($request->has('old_file')){
            //     if(is_file(Storage::url($request->old_file)))
            //     Storage::delete($request->old_file);
            // }
        }
        Download::where('id',$id)->update($data);
        return redirect()->route('admin.downloads.index')->with('success','Download has been updated successfully.');
        // return redirect()->route('manageAlbums')->with('success', 'Album updated successfully');
    }

    public function destroy($id)
    {
        Download::where('id',$id)->delete();
        return redirect()->route('admin.downloads.index')->with('success','Download has been deleted successfully.');
    }

    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }

    // public function getAlbumImages($albumId){
    //     $photos = Photo::where('album_id',$albumId)->orderBy('clickedDate', 'asc')->get();
    //     $album = DownloadCategory::where('id',$albumId)->first();
    //     return view('photos.managePhotos',compact('photos','albumId','album'));
    // }


    // public function getAllAlbums():JsonResponse{
    //     $albums = DownloadCategory::all();
    //     return response()->json(['albums'=>$albums]);
    // }

    // public function getAlbumWithPhotos($albumId):JsonResponse{
    //     $album = DownloadCategory::with('photos')->find($albumId);
    //     if(!$album){
    //         return response()->json(['error'=>'Album not found']);
    //     }
    //     return response()->json(['album'=>$album]);
    // }
}
