<?php

namespace App\Http\Controllers\Backend;

use App\Helpers\GeneralHelper;
use App\Http\Controllers\Controller;

use App\Models\Event;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Illuminate\View\View;
use Illuminate\Support\Facades\Storage;
use App\Traits\UploadTrait;
use Illuminate\Contracts\Cache\Store;

class EventController extends Controller
{

    use UploadTrait;

    private $fileUploadLocation = '/uploads/events/';
        /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(): View
    {
        $data['data'] = Event::get();
        return view('backend.events.index', $data);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('backend.events.add');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = request()->validate([
            'title' => 'required',
            'description' => 'nullable',
            'map_link' => 'nullable',
            'event_date' => 'nullable',
            'event_time' => 'nullable',
            'image' => 'required|mimes:jpg,bmp,png,jpeg',
            'location' => 'required',
            'sort_order' => '',
        ]);
        $data['sort_order'] = request()->sort_order??1;
        $data['slug'] = $this->generate_slug($data['title']);
        
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }

        if (request()->hasFile('image')) {
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation); 
        }
        Event::create($data);

        // if (request()->has('again')) {
        //     return redirect('/myadmin/generalsettings/addGeneralsettings')->with('success', 'Data has been inserted.');
        // } else {
                return redirect()->route('admin.events.index')->with('success','New Event created successfully.');
        // }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    // public function show(News $news)
    // {
    //     return view('backend.news.show',['data' => $news]);

    // }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Event $event)
    {
        // dd($event);
        return view('backend.events.edit',['data' => $event]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = request()->validate([
            'title' => 'required',
            'description' => 'nullable',
            'map_link' => 'nullable',
            'event_date' => 'nullable',
            'event_time' => 'nullable',
            'location' => 'required',
            'sort_order' => '',
        ]);
        $data['slug'] = $this->generate_slug($data['title']);

        if (request()->has('sort_order')) {
            $data['sort_order'] = request()->sort_order??1;
        }
        
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }

        if (request()->hasFile('image')) {
            $request->validate([
                'image' => 'mimes:jpg,bmp,png,jpeg'
            ]);
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation); 
            // if($request->has('old_logo')){
            //     if(file_exists(Storage::url($request->old_logo)))
            //     Storage::delete($request->old_logo);
            // }
        }

        $update = Event::where('id', $id)->update($data);
        return redirect()->route('admin.events.index')->with('success','Events updated successfully.');

        // if (request()->has('again') && request()->input('again') == '1') {
        //     return redirect('/myadmin/generalsettings/editGeneralsettings/' . $id)->with('success', 'Data has been Updated.');
        // } else {
        //     return redirect('/myadmin/generalsettings')->with('success', 'Data has been Updated.');
        // }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Event $event)
    {
        //     if(file_exists(Storage::url($news->image)))
        //     Storage::delete($news->image);
        $event->delete();
        return redirect()->route('admin.events.index')->with('success','Event has been deleted successfully.');
    }

    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }
}
