<?php

namespace App\Http\Controllers\Backend;

use App\Helpers\GeneralHelper;
use App\Http\Controllers\Controller;

use App\Models\News;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Illuminate\View\View;
use Illuminate\Support\Facades\Storage;
use App\Traits\UploadTrait;
use Illuminate\Contracts\Cache\Store;

class NewsController extends Controller
{

    use UploadTrait;

    private $fileUploadLocation = '/uploads/news/';
        /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(): View
    {
        $data['data'] = News::get();
        return view('backend.news.index', $data);
    }

   

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('backend.news.add');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = request()->validate([
            'title' => 'required',
            'description' => 'required',
            'author' => 'sometimes',
            'image' => 'required|mimes:jpg,bmp,png,jpeg',
            'published_date' => 'required',
            'sort_order' => '',
        ]);
        $data['sort_order'] = request()->sort_order??1;
        $data['slug'] = $this->generate_slug($data['title']);

        if (request()->has('show_form')) {
            $data['show_form'] = @request()->show_form?'1':'0';
        }else{
            $data['show_form'] = 0;
        }
        
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }

        if (request()->hasFile('image')) {
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation); 
        }
        News::create($data);

        // if (request()->has('again')) {
        //     return redirect('/myadmin/generalsettings/addGeneralsettings')->with('success', 'Data has been inserted.');
        // } else {
                return redirect()->route('admin.news.index')->with('success','New News created successfully.');
        // }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(News $news)
    {
        return view('backend.news.show',['data' => $news]);

    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(News $news)
    {
        return view('backend.news.edit',['data' => $news]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = request()->validate([
            'title' => 'required',
            'description' => 'required',
            'author' => 'sometimes',
            'published_date' => 'required',
            'sort_order' => '',
        ]);
        $data['slug'] = $this->generate_slug($data['title']);

        if (request()->has('sort_order')) {
            $data['sort_order'] = request()->sort_order??1;
        }
        
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }

        if (request()->hasFile('image')) {
            $request->validate([
                'image' => 'mimes:jpg,bmp,png,jpeg'
            ]);
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation); 
            // if($request->has('old_logo')){
            //     if(file_exists(Storage::url($request->old_logo)))
            //     Storage::delete($request->old_logo);
            // }
        }

        $update = News::where('id', $id)->update($data);
        return redirect()->route('admin.news.index')->with('success','News updated successfully.');

        // if (request()->has('again') && request()->input('again') == '1') {
        //     return redirect('/myadmin/generalsettings/editGeneralsettings/' . $id)->with('success', 'Data has been Updated.');
        // } else {
        //     return redirect('/myadmin/generalsettings')->with('success', 'Data has been Updated.');
        // }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(News $news)
    {
        //     if(file_exists(Storage::url($news->image)))
        //     Storage::delete($news->image);
        $news->delete();
        return redirect()->route('admin.news.index')->with('success','News has been deleted successfully.');
    }

    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }
}
