<?php

namespace App\Http\Controllers\Backend;

use App\Helpers\GeneralHelper;
use App\Http\Controllers\Controller;

use App\Models\OrganizationStructure;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Illuminate\View\View;
use Illuminate\Support\Facades\Storage;
use App\Traits\UploadTrait;
use Illuminate\Contracts\Cache\Store;

class OrganizationStructureController extends Controller
{

    use UploadTrait;

    private $fileUploadLocation = '/uploads/organization/';
        /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    public function index(): View
    {
        $data['data'] = OrganizationStructure::first();
        if (!empty($data['data'])) {
            $data['action'] = "Edit";
        } else {
            $data['action'] = "Add";
        }
        return view('backend.organization.formOrganization', $data);
    }

   

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    // public function create()
    // {
    //     $data['action'] = "Add";
    //     return view('admin.organization.formOrganization', $data);
    // }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = request()->validate([
            'title' => 'required',
            'description' => 'required',
            'image' => 'required|mimes:jpg,bmp,png,jpeg'
        ]);
        $data['slug'] = $this->generate_slug($data['title']);
  
        $data['active'] = 1;

        if (request()->hasFile('image')) {
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation); 
        }
        OrganizationStructure::create($data);

        // if (request()->has('again')) {
        //     return redirect('/myadmin/generalsettings/addGeneralsettings')->with('success', 'Data has been inserted.');
        // } else {
            return redirect()->route('admin.organization-structure.index')->with('success','Organization Structure created successfully.');
        // }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    // public function edit($id)
    // {
    //     return view('backend.organization.edit',['data' => $organizationStructure]);
    // }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = request()->validate([
            'title' => 'required',
            'description' => 'required',
        ]);
        
        $data['slug'] = $this->generate_slug($data['title']);

        $data['active'] = 1;

        if (request()->hasFile('image')) {
            $request->validate([
                'image' => 'mimes:jpg,bmp,png,jpeg'
            ]);
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation); 
            // if($request->has('old_logo')){
            //     if(file_exists(Storage::url($request->old_logo)))
            //     Storage::delete($request->old_logo);
            // }
        }

        $update = OrganizationStructure::where('id', $id)->update($data);
        return redirect()->route('admin.organization-structure.index')->with('success','Organization updated successfully.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    // public function destroy(News $news)
    // {
    //     //     if(file_exists(Storage::url($news->image)))
    //     //     Storage::delete($news->image);
    //     $news->delete();
    //     return redirect()->route('admin.organization.index')->with('success','News has been deleted successfully.');
    // }

    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }
}
