<?php

namespace App\Http\Controllers\Backend;

use App\Helpers\GeneralHelper;
use App\Http\Controllers\Controller;
use App\Models\Album;
use Illuminate\Support\Str;
use App\Models\Photo;
use App\Traits\UploadTrait;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class PhotoController extends Controller
{
    use UploadTrait;

    private $fileUploadLocation = '/uploads/photo/';

    public function index($id){
        $data['album'] = Album::where('id',$id)->first();
        return view('backend.photo.index',$data);
    }

    public function create($id){
        $data['albums'] = Album::get();

        $data['album'] = Album::where('id',$id)->first();
        return view('backend.photo.add',$data);
    }

    public function store(Request $request,$id){
        $data = $request->validate([
            'image'=>'required|nullable|mimes:jpeg,png,jpg,gif,svg|max:6000',
            'album_id' => 'required|exists:albums,id'
        ]);
        $data['sort_order'] = request()->sort_order??1;
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        if (request()->has('is_cover')) {
            $data['is_cover'] = @request()->is_cover?'1':'0';
        }else{
            $data['is_cover'] = 0;
        }
        if (request()->hasFile('image')) {
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation);
        }

        Photo::create($data);

        return redirect()->route('admin.photo.index',$request->album_id)->with('success','Image Posted successfully');

    }

    public function edit($photo,$id){
        $data['albums'] = Album::get();

        $data['data'] = Photo::where('id',$photo)->first();
        $data['album'] = Album::where('id',$id)->first();
        return view('backend.photo.edit',$data);
    }

    public function update(Request $request,$id)
    {
        $data = $request->validate([
            'album_id' => 'required|exists:albums,id',
            'image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg'
        ]);
        $data['sort_order'] = request()->sort_order??1;
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        if (request()->has('is_cover')) {
            $data['is_cover'] = @request()->is_cover?'1':'0';
        }else{
            $data['is_cover'] = 0;
        }

        if (request()->hasFile('image')) {
            $data['image'] = $this->uploadFile(request()->file('image'), $this->fileUploadLocation);
              // if($request->has('od_file')){
            //     if(file_exists(Storage::url($request->old_file)))
            //     Storage::delete($request->old_file);
            // }
        }
        Photo::where('id',$id)->update($data);
        return redirect()->route('admin.photo.index',$request->album_id)->with('success','Photo update successfully');
        // return redirect()->route('manageAlbums')->with('success', 'Album updated successfully');
    }

    public function destroy($id)
    {
        $data = Photo::find($id)->first();
        $album_id = $data->album_id;
        $data->delete();
        return redirect()->route('admin.photo.index',$album_id)->with('success','Photo deleted successfully');
    }


    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }
}
