<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Models\Service;
use App\Helpers\GeneralHelper;
use Illuminate\Http\Request;
use App\Traits\UploadTrait;
use Illuminate\Support\Facades\Storage;
use Illuminate\View\View;

class ServiceController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    use UploadTrait;

    private $fileUploadLocation = '/uploads/services/';
        /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(): View
    {
        $data['data'] = Service::get();
        return view('backend.services.index', $data);
    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('backend.services.add');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $data = $request->validate([
            'title' => 'required|string|max:255',
            'first_description' => 'required',
            'second_description' => 'sometimes|nullable',
            'first_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'second_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'third_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'fourth_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'sort_order' => 'sometimes|nullable'
        ]);

        $data['sort_order'] = request()->sort_order??1;
        $data['slug'] = $this->generate_slug($data['title']);
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        if (request()->hasFile('first_image')) {
            $data['first_image'] = $this->uploadFile(request()->file('first_image'), $this->fileUploadLocation);
        }
        if (request()->hasFile('second_image')) {
            $data['second_image'] = $this->uploadFile(request()->file('second_image'), $this->fileUploadLocation);
        }
        if (request()->hasFile('third_image')) {
            $data['third_image'] = $this->uploadFile(request()->file('third_image'), $this->fileUploadLocation);
        }
        if (request()->hasFile('fourth_image')) {
            $data['fourth_image'] = $this->uploadFile(request()->file('fourth_image'), $this->fileUploadLocation);
        }
        Service::create($data);
                return redirect()->route('admin.services.index')->with('success','New Services created successfully.');
    }

    /**
     * Display the specified resource.
     */
    public function show(Service $service)
    {
        return view('backend.services.show',['data' => $service]);
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Service $service)
    {
        return view('backend.services.edit',['data' => $service]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        $data = $request->validate([
            'title' => 'required|string|max:255',
            'first_description' => 'required',
            'second_description' => 'sometimes|nullable',
            'first_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'second_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'third_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'fourth_image' => 'sometimes|nullable|mimes:jpg,bmp,png,jpeg',
            'sort_order' => 'sometimes|nullable'
        ]);
        $data['sort_order'] = request()->sort_order??1;
        $data['slug'] = $this->generate_slug($data['title']);

        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }

        if (request()->hasFile('first_image')) {
            $data['first_image'] = $this->uploadFile(request()->file('first_image'), $this->fileUploadLocation);
              // if($request->has('old_file')){
            //     if(is_file(Storage::url($request->old_file)))
            //     Storage::delete($request->old_file);
            // }
        }
        if (request()->hasFile('second_image')) {
            $data['second_image'] = $this->uploadFile(request()->file('second_image'), $this->fileUploadLocation);
        }
        if (request()->hasFile('third_image')) {
            $data['third_image'] = $this->uploadFile(request()->file('third_image'), $this->fileUploadLocation);
        }
        if (request()->hasFile('fourth_image')) {
            $data['fourth_image'] = $this->uploadFile(request()->file('fourth_image'), $this->fileUploadLocation);
        }

        $update = Service::where('id', $id)->update($data);
        return redirect()->route('admin.services.index')->with('success','Services updated successfully.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Service $service)
    {
        // if (Storage::exists($this->fileUploadLocation . $service->image)) {
        //     Storage::delete($this->fileUploadLocation . $service->image);
        // }

        $service->delete();
        return redirect()->route('admin.services.index')->with('success','Service has been deleted successfully.');
    }
    private function generate_slug($str)
    {
        $pageName = GeneralHelper::slugify($str);
        return $pageName;
    }
}
