<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Models\Testimonial;
use App\Traits\UploadTrait;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\View\View;

class TestimonialController extends Controller
{
    use UploadTrait;

    private $fileUploadLocation = '/uploads/testimonials/';
        /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(): View
    {
        $data['data'] = Testimonial::get();
        return view('backend.testimonials.index', $data);
    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('backend.testimonials.add');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $data = request()->validate([
            'name' => 'required',
            'description' => 'required',
            'image' => 'required|mimes:jpg,bmp,png,jpeg',
        ]);
        $data['sort_order'] = request()->sort_order??1;
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        if (request()->hasFile('image')) {
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation);
        }
        Testimonial::create($data);
                return redirect()->route('admin.testimonials.index')->with('success','New Testimonials created successfully.');
    }

    /**
     * Display the specified resource.
     */
    public function show(Testimonial $testimonial)
    {
        return view('backend.testimonials.show',['data' => $testimonial]);
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Testimonial $testimonial)
    {
        return view('backend.testimonials.edit',['data' => $testimonial]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        $data = request()->validate([
            'name' => 'required',
            'description' => 'required',
        ]);
        $data['sort_order'] = request()->sort_order??1;
        if (request()->has('active')) {
            $data['active'] = @request()->active?'1':'0';
        }else{
            $data['active'] = 0;
        }
        if (request()->hasFile('image')) {
            $request->validate([
                'image' => 'mimes:jpg,bmp,png,jpeg'
            ]);
            $data['image'] = $this->uploadOne(request()->file('image'), $this->fileUploadLocation);

        }

        $update = Testimonial::where('id', $id)->update($data);
        return redirect()->route('admin.testimonials.index')->with('success','Testimonials updated successfully.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Testimonial $testimonial)
    {
        // if (Storage::exists($this->fileUploadLocation . $testimonial->image)) {
        //     Storage::delete($this->fileUploadLocation . $testimonial->image);
        // }

        $testimonial->delete();
        return redirect()->route('admin.testimonials.index')->with('success','Testimonial has been deleted successfully.');
    }

}
