<?php

namespace App\Traits;

// use App\Jobs\UploadFileJob;
use Exception;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Intervention\Image\Laravel\Facades\Image;
use Intervention\Image\ImageManager;
use Intervention\Image\Drivers\Gd\Driver;
use Intervention\Image\Encoders\WebpEncoder;

trait UploadTrait
{
    /* (Sangharsha) Note: We will probably move on with intervention, so the question is how do we go about it?
    create a seperate column or create a seperate specific folder(thumbnails) which will mirror the
    folder structure of original images. I would prefer the later option.
    refer:::
    https://www.nicesnippets.com/blog/laravel-9-image-resize-upload-with-intervention-image
    https://onlinecode.org/laravel-9-image-resize-and-upload-example/
     */

    /* Structure for $folder="/folder/folder" */
    /* Used in PetService for reference */
    public function uploadOne(UploadedFile $uploaded_file, $folder = '', $disk = 'public', $filename = null)
    {
        try {
            $name = !is_null($filename) ? $filename : time() . Str::random(5);
            $folder_name = $this->prepFolder($folder);
            $full_name = $name . '.' . $uploaded_file->getClientOriginalExtension();
            if ($uploaded_file->getClientMimeType() == 'image/heic' || $uploaded_file->getClientMimeType() == 'image/heif' || $uploaded_file->getClientMimeType() == 'application/octet-toFilePointer') {
                $upload = $uploaded_file->storeAs($folder_name, $full_name, $disk);
            } else {
                //This works too
                // $manager = new ImageManager(new Driver());
                // $image = $manager->read($uploaded_file)->resize(1500, 1500, function ($constraint) {
                //     $constraint->aspectRatio();
                //     $constraint->upsize();
                // })->encode(new WebpEncoder(quality: 90));
                $image = Image::read($uploaded_file)->encode(new WebpEncoder(quality: 70));
                $upload = Storage::disk($disk)->put($folder_name . $full_name, $image);
            }

            // $image = Image::make($uploaded_file)->resize(1500, 1500, function ($constraint) {
            //     $constraint->aspectRatio();
            //     $constraint->upsize();
            // })->toFilePointer('jpeg', 80);
            // $upload = Storage::disk($disk)->put($folder_name . $full_name, $image);

            $file = ltrim($folder_name . $full_name, '//');
            // $jobData = [
            //     'file' => $file,
            //     'folder' => $folder_name,
            //     'disk' => $disk,
            //     'full_name' => $full_name,
            // ];
            // $this->resizeImage($file, $folder, $disk, $full_name);
            // $jobToDispatch = (new UploadFileJob($jobData))->delay(Carbon::now()->addSeconds(1));
            // dispatch($jobToDispatch);
            return $file;
        } catch (Exception $e) {
            Log::error($e->getMessage());
        }
    }

    public function uploadFile(UploadedFile $uploaded_file, $folder = '', $disk = 'public', $filename = null)
    {
        try {
            $name = !is_null($filename) ? $filename : time() . Str::random(5);
            $folder_name = $this->prepFolder($folder);
            $full_name = $name . '.' . $uploaded_file->getClientOriginalExtension();

            $upload = $uploaded_file->storeAs($folder_name, $full_name, $disk);
         
            // $image = Image::make($uploaded_file)->resize(1500, 1500, function ($constraint) {
            //     $constraint->aspectRatio();
            //     $constraint->upsize();
            // })->toFilePointer('jpeg', 80);
            // $upload = Storage::disk($disk)->put($folder_name . $full_name, $image);

            $file = ltrim($folder_name . $full_name, '//');
            // $jobData = [
            //     'file' => $file,
            //     'folder' => $folder_name,
            //     'disk' => $disk,
            //     'full_name' => $full_name,
            // ];
            // $this->resizeImage($file, $folder, $disk, $full_name);
            // $jobToDispatch = (new UploadFileJob($jobData))->delay(Carbon::now()->addSeconds(1));
            // dispatch($jobToDispatch);
            return $file;
        } catch (Exception $e) {
            Log::error($e->getMessage());
        }
    }

    public function uploadImage(UploadedFile $uploaded_file, $folder = '', $disk = 'public', $filename = null)
    {
        try {
            $name = !is_null($filename) ? $filename : time() . Str::random(5);
            $folder_name = $this->prepFolder($folder);
            $full_name = $name . '.' . $uploaded_file->getClientOriginalExtension();
            $upload = $uploaded_file->storeAs($folder_name, $full_name, $disk);
            $file = ltrim($folder_name . $full_name, '//');
            return $file;
        } catch (Exception $e) {
            Log::error($e->getMessage());
        }
    }


    /* Used in SocialAccountsService for reference*/
    public function uploadFromUrl($url = '', $folder = '', $disk = 'public', $filename = null)
    {
        if (!$url) {
            return null;
        }
        try {
            // $uri_segments = explode('?', $url);
            // $info = pathinfo($uri_segments[0]);
            // $name = !is_null($filename) ? $filename : time() . Str::random(5);
            // $full_name = $name . '.png';
            // $image = Image::make($uri_segments[0])->toFilePointer();
            // $folder_name = $this->prepFolder($folder);
            // $path = Storage::disk($disk)->put($folder_name . $full_name, $image);
            // $file = ltrim($folder_name . $full_name, '/');
            $imageData = file_get_contents($url);
            $name = !is_null($filename) ? $filename : time() . Str::random(5);
            $full_name = $name . '.png';
            $image = Image::read($imageData)->toFilePointer();
            $folder_name = $this->prepFolder($folder);
            $path = Storage::disk($disk)->put($folder_name . $full_name, $image);
            $file = ltrim($folder_name . $full_name, '/');
            $jobData = [
                'file' => $file,
                'folder' => $folder_name,
                'disk' => $disk,
                'full_name' => $full_name,
            ];
            $jobToDispatch = (new UploadFileJob($jobData))->delay(Carbon::now()->addSeconds(1));
            dispatch($jobToDispatch);
            return $file;
        } catch (Exception $e) {
            return null;
        }
    }

    public function uploadBase64($image_64, $folder = '', $disk = 'public', $filename = null)
    {
        try {
            $ext = explode('/', mime_content_type($image_64))[1];
            $extension = explode('+', $ext)[0];
            $name = !is_null($filename) ? $filename : time() . Str::random(5);
            $full_name = $name . '.' . $extension;
            $folder_name = $this->prepFolder($folder);
            $image = Image::make($image_64)->orientate()->resize(1500, 1500, function ($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->toFilePointer($extension, 80);
            $upload = Storage::disk($disk)->put($folder_name . $full_name, $image);
            $file = ltrim($folder_name . $full_name, '/');
            $jobData = [
                'file' => $file,
                'folder' => $folder_name,
                'disk' => $disk,
                'full_name' => $full_name,
            ];
            $jobToDispatch = (new UploadFileJob($jobData))->delay(Carbon::now()->addSeconds(1));
            dispatch($jobToDispatch);
            return $file;
        } catch (Exception $e) {
            return null;
        }
    }


    public function removeUploadBase64($image_path)
    {
        // @Sangarshan need to append config and also form mobile image delete
        if (File::exists(public_path('/storage/' . $image_path))) {
            unlink(public_path('/storage/' . $image_path));
        }
    }
    //Call image url save or append \storage somewhere
    // also folders for mobile and thumbs appends somewhere
    private function resizeImage($file, $folder, $disk, $full_name)
    {
        try {
            $path = asset(config('uploads.uploads') . $file);
            $image = Image::read($path)->resize(200, 200, function ($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->encode(new WebpEncoder(quality: 70));
            $path = Storage::disk($disk)->put('mobile' . $folder . $full_name, $image);
            return $path;
        } catch (Exception $e) {
            Log::error($e->getMessage());
        }
    }

    private function prepFolder($folder)
    {
        if ($folder != '') {
            if ($folder[1] != '/') {
                $folder = '/' . $folder;
            }
            if ($folder[-1] != '/') {
                $folder = $folder . '/';
            }
        } else {
            $folder = '/';
        }
        return $folder;
    }

    private function readFileInChunks($filePath)
    {
        $handle = fopen($filePath, 'r');
        $contents = '';
        while (!feof($handle)) {
            $contents .= fread($handle, 8192);
        }
        fclose($handle);
        return $contents;
    }
}